/* Copyright 2013, Unpublished Work of Technologic Systems
 * All Rights Reserved.
 *
 * THIS WORK IS AN UNPUBLISHED WORK AND CONTAINS CONFIDENTIAL,
 * PROPRIETARY AND TRADE SECRET INFORMATION OF TECHNOLOGIC SYSTEMS.
 * ACCESS TO THIS WORK IS RESTRICTED TO (I) TECHNOLOGIC SYSTEMS 
 * EMPLOYEES WHO HAVE A NEED TO KNOW TO PERFORM TASKS WITHIN THE SCOPE
 * OF THEIR ASSIGNMENTS AND (II) ENTITIES OTHER THAN TECHNOLOGIC
 * SYSTEMS WHO HAVE ENTERED INTO APPROPRIATE LICENSE AGREEMENTS.  NO
 * PART OF THIS WORK MAY BE USED, PRACTICED, PERFORMED, COPIED, 
 * DISTRIBUTED, REVISED, MODIFIED, TRANSLATED, ABRIDGED, CONDENSED, 
 * EXPANDED, COLLECTED, COMPILED, LINKED, RECAST, TRANSFORMED, ADAPTED
 * IN ANY FORM OR BY ANY MEANS, MANUAL, MECHANICAL, CHEMICAL, 
 * ELECTRICAL, ELECTRONIC, OPTICAL, BIOLOGICAL, OR OTHERWISE WITHOUT
 * THE PRIOR WRITTEN PERMISSION AND CONSENT OF TECHNOLOGIC SYSTEMS.
 * ANY USE OR EXPLOITATION OF THIS WORK WITHOUT THE PRIOR WRITTEN
 * CONSENT OF TECHNOLOGIC SYSTEMS COULD SUBJECT THE PERPETRATOR TO
 * CRIMINAL AND CIVIL LIABILITY.
 */

/* This is an 8 bit wishbone slave with 2 registers.  The first register is
 * for data.  A write to it places a byte in the FIFO.  A read retrieves a
 * byte from the FIFO.  Up to 1024 bytes can be stored.
 *
 * The second register is a status register.
 */

module wb_fifo_1kbyte(
  wb_clk_i,
  wb_rst_i,

  wb1_cyc_i,
  wb1_stb_i,
  wb1_we_i,
  wb1_adr_i,
  wb1_dat_i,
  wb1_dat_o,
  wb1_ack_o,

  wb2_cyc_i,
  wb2_stb_i,
  wb2_we_i,
  wb2_adr_i,
  wb2_dat_i,
  wb2_dat_o,
  wb2_ack_o,

  wb3_cyc_i,
  wb3_stb_i,
  wb3_we_i,
  wb3_adr_i,
  wb3_dat_i,
  wb3_dat_o,
  wb3_ack_o,

  wb4_cyc_i,
  wb4_stb_i,
  wb4_we_i,
  wb4_adr_i,
  wb4_dat_i,
  wb4_dat_o,
  wb4_ack_o
);

input wb_clk_i, wb_rst_i;

input wb1_cyc_i, wb1_stb_i, wb1_we_i;
input [31:0] wb1_adr_i;
input [7:0] wb1_dat_i;
output [7:0] wb1_dat_o;
output wb1_ack_o;

input wb2_cyc_i, wb2_stb_i, wb2_we_i;
input [31:0] wb2_adr_i;
input [7:0] wb2_dat_i;
output [7:0] wb2_dat_o;
output wb2_ack_o;

input wb3_cyc_i, wb3_stb_i, wb3_we_i;
input [31:0] wb3_adr_i;
input [7:0] wb3_dat_i;
output [7:0] wb3_dat_o;
output wb3_ack_o;

input wb4_cyc_i, wb4_stb_i, wb4_we_i;
input [31:0] wb4_adr_i;
input [7:0] wb4_dat_i;
output [7:0] wb4_dat_o;
output wb4_ack_o;

wire wb_cyc_i, wb_stb_i, wb_we_i;
wire [31:0] wb_adr_i;
wire [7:0] wb_dat_i;
reg [7:0] wb_dat_o;
reg wb_ack_o;

wb_arbiter arbitercore(
  .wb_clk_i(wb_clk_i),

  .wbowner_dat_o(wb_dat_i),
  .wbowner_dat_i(wb_dat_o),
  .wbowner_adr_o(wb_adr_i),
  .wbowner_cyc_o(wb_cyc_i),
  .wbowner_stb_o(wb_stb_i),
  .wbowner_we_o(wb_we_i),
  .wbowner_ack_i(wb_ack_o),

  .wb1_adr_i(wb1_adr_i),
  .wb1_dat_i(wb1_dat_i),
  .wb1_dat_o(wb1_dat_o),
  .wb1_cyc_i(wb1_cyc_i),
  .wb1_stb_i(wb1_stb_i),
  .wb1_we_i(wb1_we_i),
  .wb1_ack_o(wb1_ack_o),

  .wb2_adr_i(wb2_adr_i),
  .wb2_dat_i(wb2_dat_i),
  .wb2_dat_o(wb2_dat_o),
  .wb2_cyc_i(wb2_cyc_i),
  .wb2_stb_i(wb2_stb_i),
  .wb2_we_i(wb2_we_i),
  .wb2_ack_o(wb2_ack_o),

  .wb3_adr_i(wb3_adr_i),
  .wb3_dat_i(wb3_dat_i),
  .wb3_dat_o(wb3_dat_o),
  .wb3_cyc_i(wb3_cyc_i),
  .wb3_stb_i(wb3_stb_i),
  .wb3_we_i(wb3_we_i),
  .wb3_ack_o(wb3_ack_o),

  .wb4_adr_i(wb4_adr_i),
  .wb4_dat_i(wb4_dat_i),
  .wb4_dat_o(wb4_dat_o),
  .wb4_cyc_i(wb4_cyc_i),
  .wb4_stb_i(wb4_stb_i),
  .wb4_we_i(wb4_we_i),
  .wb4_ack_o(wb4_ack_o)
);

wire [7:0] dout;
wire full, empty;
wire [10:0] data_count;
reg wr_en, rd_en;

fifo_1kx1byte fifocore(
  .clk(wb_clk_i),
  .srst(wb_rst_i),
  .din(wb_dat_i),
  .wr_en(wr_en), // input wr_en
  .rd_en(rd_en), // input rd_en
  .dout(dout), // output [7 : 0] dout
  .full(full), // output full
  .empty(empty), // output empty
  .data_count(data_count) // output [10 : 0] data_count
);

wire dat_cyc = wb_cyc_i && wb_stb_i && !wb_adr_i[0];

reg [5:0] status;
always @(*) begin
  if (data_count[10:5] == 6'd0) status = {1'b0, data_count[4:0]};
  else status = {1'b1, data_count[10:6]};
end

always @(posedge wb_clk_i or posedge wb_rst_i) begin
  if (wb_rst_i) begin
    wb_ack_o <= 1'b0;
    wb_dat_o <= 8'h00;
    wr_en <= 1'b0;
    rd_en <= 1'b0;
  end else begin
    wb_ack_o <= wb_cyc_i && wb_stb_i && !wb_ack_o;
    if (dat_cyc) wb_dat_o <= dout;
    else wb_dat_o <= {empty, full, status};
    wr_en <= dat_cyc && wb_we_i && !wb_ack_o;
    rd_en <= dat_cyc && !wb_we_i && !wb_ack_o;
  end
end

endmodule

