/* Copyright 2012, Unpublished Work of Technologic Systems
 * All Rights Reserved.
 *
 * THIS WORK IS AN UNPUBLISHED WORK AND CONTAINS CONFIDENTIAL,
 * PROPRIETARY AND TRADE SECRET INFORMATION OF TECHNOLOGIC SYSTEMS.
 * ACCESS TO THIS WORK IS RESTRICTED TO (I) TECHNOLOGIC SYSTEMS 
 * EMPLOYEES WHO HAVE A NEED TO KNOW TO PERFORM TASKS WITHIN THE SCOPE
 * OF THEIR ASSIGNMENTS AND (II) ENTITIES OTHER THAN TECHNOLOGIC
 * SYSTEMS WHO HAVE ENTERED INTO APPROPRIATE LICENSE AGREEMENTS.  NO
 * PART OF THIS WORK MAY BE USED, PRACTICED, PERFORMED, COPIED, 
 * DISTRIBUTED, REVISED, MODIFIED, TRANSLATED, ABRIDGED, CONDENSED, 
 * EXPANDED, COLLECTED, COMPILED, LINKED, RECAST, TRANSFORMED, ADAPTED
 * IN ANY FORM OR BY ANY MEANS, MANUAL, MECHANICAL, CHEMICAL, 
 * ELECTRICAL, ELECTRONIC, OPTICAL, BIOLOGICAL, OR OTHERWISE WITHOUT
 * THE PRIOR WRITTEN PERMISSION AND CONSENT OF TECHNOLOGIC SYSTEMS.
 * ANY USE OR EXPLOITATION OF THIS WORK WITHOUT THE PRIOR WRITTEN
 * CONSENT OF TECHNOLOGIC SYSTEMS COULD SUBJECT THE PERPETRATOR TO
 * CRIMINAL AND CIVIL LIABILITY.
 */

/* These modules are necessary because the PCIe DMA channels can only emit
 * 32 bit wishbone cycles, but we need to use them to stream data from 16 bit
 * or 8 bit slaves.
 *
 * The incr_i input should be driven to 1 for normal incrementing cycles.  For
 * non-incrementing cycles, addresses are shifted by 1 or 2 bits so that 
 * you can camp on any 8 bit or 16 bit address.
 */

module wb_32to16_adapter (
  wb_clk_i,
  wb_rst_i,

  wb_cyc_i,
  wb_stb_i,
  wb_we_i,
  wb_adr_i,
  wb_dat_i,
  wb_sel_i,
  wb_dat_o,
  wb_ack_o,

  wbm_cyc_o,
  wbm_stb_o,
  wbm_dat_o,
  wbm_adr_o,
  wbm_we_o,
  wbm_dat_i,
  wbm_ack_i,
  wbm_sel_o,

  incr_i
);

input wb_clk_i, wb_rst_i, wb_cyc_i, wb_stb_i, wb_we_i;
input [31:0] wb_adr_i;
input [31:0] wb_dat_i;
input [3:0] wb_sel_i;
output [31:0] wb_dat_o;
output wb_ack_o;

output wbm_cyc_o;
output wbm_stb_o;
output [15:0] wbm_dat_o;
output [31:0] wbm_adr_o;
output wbm_we_o;
input [15:0] wbm_dat_i;
input wbm_ack_i;
output [1:0] wbm_sel_o;

input incr_i;

reg [15:0] dat;
reg phase;

assign wb_dat_o = {wbm_dat_i, dat};
assign wb_ack_o = phase ? wbm_ack_i : 1'b0;

assign wbm_cyc_o = wb_cyc_i;
assign wbm_stb_o = wb_stb_i;
assign wbm_dat_o = phase ? wb_dat_i[15:0] : wb_dat_i[31:16];
assign wbm_adr_o[31:1] = incr_i ? {wb_adr_i[31:2], phase} :
                                  {1'b0, wb_adr_i[31:2]};
assign wbm_adr_o[0] = 1'b0;
assign wbm_we_o = wb_we_i;
assign wbm_sel_o = 2'b11;

always @(posedge wb_clk_i or posedge wb_rst_i) begin
  if (wb_rst_i) begin
    phase <= 1'b0;
    dat <= 16'd0;
  end else begin
    if (wbm_ack_i) phase <= !phase;
    if (wbm_ack_i && !phase) dat <= wbm_dat_i;
  end
end

endmodule


module wb_32to8_adapter (
  wb_clk_i,
  wb_rst_i,

  wb_cyc_i,
  wb_stb_i,
  wb_we_i,
  wb_adr_i,
  wb_dat_i,
  wb_sel_i,
  wb_dat_o,
  wb_ack_o,

  wbm_cyc_o,
  wbm_stb_o,
  wbm_dat_o,
  wbm_adr_o,
  wbm_we_o,
  wbm_dat_i,
  wbm_ack_i,

  incr_i,
  phase_o
);

input wb_clk_i, wb_rst_i, wb_cyc_i, wb_stb_i, wb_we_i;
input [31:0] wb_adr_i;
input [31:0] wb_dat_i;
input [3:0] wb_sel_i;
output [31:0] wb_dat_o;
output wb_ack_o;

output wbm_cyc_o;
output wbm_stb_o;
output [7:0] wbm_dat_o;
output [31:0] wbm_adr_o;
output wbm_we_o;
input [7:0] wbm_dat_i;
input wbm_ack_i;

input incr_i;
output [1:0] phase_o;

reg [23:0] dat;
reg [1:0] phase;
reg [7:0] wdat;
assign phase_o = phase;

assign wb_dat_o = {wbm_dat_i, dat};
assign wb_ack_o = (phase == 2'b11) ? wbm_ack_i : 1'b0;

assign wbm_cyc_o = wb_cyc_i;
assign wbm_stb_o = wb_stb_i;
assign wbm_dat_o = wdat;
assign wbm_adr_o = incr_i ? {wb_adr_i[31:2], phase} : {2'b00, wb_adr_i[31:2]};
assign wbm_we_o = wb_we_i;

always @(*) begin
  case(phase)
    2'b00: wdat = wb_dat_i[7:0];
    2'b01: wdat = wb_dat_i[15:8];
    2'b10: wdat = wb_dat_i[23:16];
    2'b11: wdat = wb_dat_i[31:24];
  endcase
end

always @(posedge wb_clk_i or posedge wb_rst_i) begin
  if (wb_rst_i) begin
    phase <= 2'b00;
    dat <= 24'd0;
  end else begin
    if (wbm_ack_i) phase <= phase + 1'b1;
    if (wbm_ack_i && (phase == 2'b00)) dat[7:0] <= wbm_dat_i;
    if (wbm_ack_i && (phase == 2'b01)) dat[15:8] <= wbm_dat_i;
    if (wbm_ack_i && (phase == 2'b10)) dat[23:16] <= wbm_dat_i;
  end
end

endmodule

